% This program is for "Tauchen's method" to approximate AR(1) process
% by 1st order Markov Chain. See Nakajima(2007) for detail.
% By Mitsuru Katagiri, 11/26/2009

format long

%-------------------------------------------------------------------------
%  Set parameters
%-------------------------------------------------------------------------

MUZ  = 0.0;     % mean of z
NOG  = ngrit;   % # of grids
    
LAMBDA = 2.0;   % the parameters to determine the width between grids.
                % Usual value is 3.0. See Nakajima(2007)

%-------------------------------------------------------------------------
%  Define some variables
%-------------------------------------------------------------------------

SIGZ = SIGE / (1 - AR1^2.0)^(0.5);  % stdev of z
UPZ  = MUZ + LAMBDA*SIGZ;           % upper bound of grid for z
LOZ  = MUZ - LAMBDA*SIGZ;      % lower bound of gird for z

VZ   = zeros(1, NOG);               % girds of z
MID  = zeros(NOG-1, 1);             % mid points of grids
MKOV = zeros(NOG,NOG);              % Markov transition matrix

%-------------------------------------------------------------------------
%  Approximate AR(1) by Markov chain
%-------------------------------------------------------------------------

% making grids for z
for j = 1:NOG
    VZ(j) = LOZ + ( (UPZ - LOZ)/(NOG-1) )*(j-1);
end

% making mid points for grids
for i = 1:NOG-1  
   MID(i) = ( VZ(i) + VZ(i+1) ) / 2.0;
end

% making transition matrix
for i = 1:NOG
    for j = 1:NOG
        
        if j == 1
            
            %MKOV(i,j) = 0.5 * (1 + erf( ( MID(j) - (1-AR1)*MUZ - AR1*VZ(i) ) / SIGE / 2^0.5 )); 
            MKOV(i,j) = cdf( 'norm' , ( MID(j) - (1-AR1)*MUZ - AR1*VZ(i) ) / SIGE ,0,1 );
%             MKOV(i,j) = cdf( 't' , ( MID(j) - (1-AR1)*MUZ - AR1*VZ(i) ) / SIGE ,3 );
        
        elseif j == NOG
            
            %MKOV(i,j) = 1 - 0.5 * (1 + erf( ( MID(j-1) - (1-AR1)*MUZ - AR1*VZ(i) ) / SIGE / 2^0.5 )); 
            MKOV(i,j) = 1-cdf( 'norm' , ( MID(j-1) - (1-AR1)*MUZ - AR1*VZ(i) ) / SIGE ,0,1 );
%             MKOV(i,j) = 1-cdf( 't' , ( MID(j-1) - (1-AR1)*MUZ - AR1*VZ(i) ) / SIGE ,3 );
        
        else

%             MKOV(i,j) = 0.5 * (1 + erf( ( MID(j)   - (1-AR1)*MUZ - AR1*VZ(i) ) / SIGE / 2^0.5 ))...
%                       - 0.5 * (1 + erf( ( MID(j-1) - (1-AR1)*MUZ - AR1*VZ(i) ) / SIGE / 2^0.5 ));
            MKOV(i,j) = cdf( 'norm' , ( MID(j)   - (1-AR1)*MUZ - AR1*VZ(i) ) / SIGE ,0,1 )...
                      - cdf( 'norm' , ( MID(j-1) - (1-AR1)*MUZ - AR1*VZ(i) ) / SIGE ,0,1 );
%             MKOV(i,j) = cdf( 't' , ( MID(j)   - (1-AR1)*MUZ - AR1*VZ(i) ) / SIGE ,3 )...
%                       - cdf( 't' , ( MID(j-1) - (1-AR1)*MUZ - AR1*VZ(i) ) / SIGE ,3 );
        end
        
    end
end


