% making grids based on VAR
% Written by Mitsuru Katagiri 2017/7/13

% Need to input the estimation results into "var_params_ti.m"&"var_params_st.m"
% befor we run "grid_var.m"

clear; clc; close all

% # of grids
ngriy  = 5;             % income growth
ngrip  = 5;             % inflation
ngrig  = 5;             % income gap
ngrit  = 5;             % trend inflation
ngriyp = ngriy*ngrip*ngrig;  % (y,p,g)

for cnt_case = 1:2 % 1: US, 2: UK
    
    disp(['cnt_case = ',num2str(cnt_case)])

for st_nonst = 3:4
    
    % -----------VAR results--------------
    switch st_nonst
      
        case 1; var_params_ti % stochastic trend
        case 2; var_params_st % trend stationary
        case 3; var_params_ti_ut87 % stochastic trend up to 1987
        case 4; var_params_ti_af87 % stochastic trend after 1987
            
    end
    
    % ------------- approximate VAR by Markov chain ---------------------
    % the last input is the width of grids (2.0 means 2-std)

    gridyp  = zeros(length(A1),ngriyp);      % grids for income and inflation
    gridyp2 = zeros(length(A1),max([ngriy; ngrip; ngrig]));    
    mcyp    = zeros(ngriyp,ngriyp); % the probability matrices 

    [mcyp(:,:),gridyp(:,:),gridyp2(:,:)] ...
        = fn_var_to_markov(eye(length(A1)),A1,A2,SIGMA,[ngriy; ngrip; ngrig],1000,1,2.0);
    
    % making grid in terms of GROSS BASIS!!
    gridy  = exp(gridyp2(1,1:ngriy));   % grid for income growth
    gridp  = exp(gridyp2(2,1:ngrip));   % grid for inflation
    gridg  = exp(gridyp2(3,1:ngrig));   % grid for income gap
    gridyp = exp(gridyp);
    
    % Making grids and transition matrix for trend inflation
    cddf

    cd 'D:\data\MKatagiri\term04\term04'
    
    switch cnt_case
        
        case 1; cd 'c1US'
        case 2; cd 'c2UK'
            
    end
    
    if     st_nonst == 2; cd '06stat'; 
    elseif st_nonst == 3; cd '14ut87'; 
    elseif st_nonst == 4; cd '15af87'; 
    end
    
    
    % store results
    dlmwrite('gridyp.txt',gridyp, 'precision', '%.6f', ...
             'newline', 'pc')   
    dlmwrite('gridy.txt',gridy, 'precision', '%.6f', ...
             'newline', 'pc')   
    dlmwrite('gridp.txt',gridp, 'precision', '%.6f', ...
             'newline', 'pc')   
    dlmwrite('gridg.txt',gridg, 'precision', '%.6f', ...
             'newline', 'pc')   
    dlmwrite('mcyp.txt',mcyp, 'precision', '%.6f', ...
             'newline', 'pc')   
    dlmwrite('gpistar.txt',gpistar, 'precision', '%.6f', ...
             'newline', 'pc')   

    dlmwrite('gridt.txt',exp(VZ/100), 'precision', '%.6f', ...
            'newline', 'pc')   
    dlmwrite('mct.txt',MKOV, 'precision', '%.6f', ...
            'newline', 'pc')
    
    cd 'D:\data\MKatagiri'
    
end
end