! subroutine for compute residuals for Euler eq.
subroutine euler_eq(c_t0, eul_re)

    use common_var

    implicit none
    
    real(8), intent(out) :: eul_re
    real(8), intent(in)  :: c_t0
    real(8) :: y_t0, p_t0, g_t0, b_t0, t_t0, r_t0, b_t1
    
    ! income and inflation
    y_t0 = gridyp(dyp,1); p_t0 = gridyp(dyp,2); g_t0 = gridyp(dyp,3) 
    
    ! budget constraint
    b_t0 = gridb(db)
    r_t0 = rmat(dyp,dt,dr)
    
    b_t1 = r_t0*(b_t0/(y_t0*p_t0*t_t0) + g_t0 - c_t0)
    
    ! interpolation for b_t+1 and r_t
    call inter_weight( b_t1, gridb, wb, jb, ngrib )
    call inter_weight( log(r_t0), log(gridr), wr, jr, ngrir )
    call inter_weight( c_t0, gridc, wc, jc, ngric )
    
    ! next periods' consumption and values
    c_t1(:,:,db) = 0.0; v_t1(:,:,db) = 0.0
    do ddb = 1, 2; do ddr = 1, 2; do ddc = 1, 2
    
        c_t1(:,:,db) = c_t1(:,:,db) + wb(ddb)*wr(ddr)*wc(ddc)*c_tf(:,:,jb(ddb),jr(ddr),jc(ddc))
        v_t1(:,:,db) = v_t1(:,:,db) + wb(ddb)*wr(ddr)*wc(ddc)*v_tf(:,:,jb(ddb),jr(ddr),jc(ddc))
    
    enddo; enddo; enddo
    
    ! AA = E[v^(1-\alp)]^(1/(1-\alp))
    AA(db) = 0.0
    do ddt = 1, ngrit
            
        AA(db) = AA(db) + mct(dt,ddt) * sum( mcyp(dyp,:)*(gridyp(:,1)*v_t1(:,ddt,db))**(1-alp_p) )
            
    enddo
            
    AA(db) = AA(db)**(1.0/(1.0-alp_p))
    
    ! euler eq. error
    eul_re = 0.0
    do ddt = 1, ngrit
            
        eul_re = eul_re + mct(dt,ddt) * sum( mcyp(dyp,:)*( &
            r_t0/gridyp(:,2) * beta_p * ( (c_t1(:,ddt,db) - hab_p*c_t0/gridyp(:,1))/(c_t0 - hab_p*gridc(dc)/gridyp(dyp,1)) )**(-sig_p) &
            *(v_t1(:,ddt,db)/AA(db))**(sig_p-alp_p) * gridyp(:,1)**(-alp_p) ) )
            
    enddo
            
    eul_re = 1.0 + phib*(b_t1/r_t0 - bst/rstar) - eul_re
    
end subroutine    
