! Calculate optimal consumption by Newton Method
subroutine opt_con 
    
!--------------------------------------------------------------------------------
!  SET PARAMETERS
!--------------------------------------------------------------------------------

    use common_var
    
    implicit none
  
    integer :: j, jjj, jjjj
    real(8) :: deriv
    real(8) :: eul_re(ngrib), eul_d(ngrib)
    real(8) :: y_t0, p_t0, g_t0, c_t0, t_t0, c_t0d, b_t0, r_t0, b_t1, lag_t0
 
!--------------------------------------------------------------------------------
!  compute optimal consumption by Newton method
!--------------------------------------------------------------------------------
    
    ! income and inflation, bond, and interest rate
    y_t0 = gridyp(dyp,1)
    p_t0 = gridyp(dyp,2) 
    g_t0 = gridyp(dyp,3) 
    r_t0 = rmat(dyp,dt,dr)
    t_t0 = gridt(dt)

    if ( ngrir > 1 ) then
        jr(1) = min0(max0(1,int(1 + (r_t0-gridr(1))/((gridr(ngrir)-gridr(1))/(ngrir-1)))),ngrir-1); jr(2) = 1 + jr(1)
        wr(1) = (gridr(jr(2)) - r_t0)/(gridr(jr(2))-gridr(jr(1))); wr(2) = 1.0 - wr(1)
    else
        wr = (/0.5, 0.5/)
        jr = (/1, 1/)
    endif
    
!$omp parallel do private(j,b_t0,c_t0,b_t1,jb,wb,jc,wc,ddb,ddc,ddr,ddt,eul_re,c_t0d,eul_d,deriv)
    do db  = 1, ngrib
    
        b_t0 = gridb(db)
        c_t0 = c_t(dyp,dt,db,dr,dc)
        
        ! Newton method to obtain optimal consumption
        do j = 1, 1000
            
            !-------------- compute Euler eq. error ------------------ 
            
            ! budget constraint
            b_t1 = r_t0*(b_t0/(y_t0*p_t0*t_t0) + g_t0 - c_t0)
            
            ! interpolation for b_t+1 and r_t
            jb(1) = min0(max0(1,int(1 + (b_t1-gridb(1))/((gridb(ngrib)-gridb(1))/(ngrib-1)))),ngrib-1); jb(2) = 1 + jb(1)
            wb(1) = (gridb(jb(2)) - b_t1)/(gridb(jb(2))-gridb(jb(1))); wb(2) = 1.0 - wb(1)
            
            if ( ngric > 1 ) then
                jc(1) = min0(max0(1,int(1 + (c_t0-gridc(1))/((gridc(ngric)-gridc(1))/(ngric-1)))),ngric-1); jc(2) = 1 + jc(1)
                wc(1) = (gridc(jc(2)) - c_t0)/(gridc(jc(2))-gridc(jc(1))); wc(2) = 1.0 - wc(1)
            else
                wc = (/0.5, 0.5/)
                jc = (/1, 1/)
            endif
            
            ! next periods' consumption and values
            c_t1(:,:,db) = 0.0; v_t1(:,:,db) = 0.0
            do ddb = 1, 2; do ddr = 1, 2; do ddc = 1, 2
            
                c_t1(:,:,db) = c_t1(:,:,db) + wb(ddb)*wr(ddr)*wc(ddc)*c_tf(:,:,jb(ddb),jr(ddr),jc(ddc))
                v_t1(:,:,db) = v_t1(:,:,db) + wb(ddb)*wr(ddr)*wc(ddc)*v_tf(:,:,jb(ddb),jr(ddr),jc(ddc))
            
            enddo; enddo; enddo
            
            ! AA = E[v^(1-\alp)]^(1/(1-\alp))
            AA(db) = 0.0
            do ddt = 1, ngrit
            
                AA(db) = AA(db) + mct(dt,ddt) * sum( mcyp(dyp,:)*(gridyp(:,1)*v_t1(:,ddt,db))**(1-alp_p) )
            
            enddo
            
            AA(db) = AA(db)**(1.0/(1.0-alp_p))
            
            ! euler eq. error
            eul_re(db) = 0.0
            do ddt = 1, ngrit
            
                eul_re(db) = eul_re(db) + mct(dt,ddt) * sum( mcyp(dyp,:)*( &
                    r_t0/(gridyp(:,2)*gridt(ddt)) * beta_p * ( (c_t1(:,ddt,db) - hab_p*c_t0/gridyp(:,1))/(c_t0 - hab_p*gridc(dc)/gridyp(dyp,1)) )**(-sig_p) &
                    *(v_t1(:,ddt,db)/AA(db))**(sig_p-alp_p) * gridyp(:,1)**(-alp_p) ) )
            
            enddo
            
            eul_re(db) = 1.0 + phib*(b_t1/r_t0 - bst/rstar) - eul_re(db)
            
            ! converge?
            if ( abs(eul_re(db)) < 1.0e-7 ) exit
            
            !-------------- compute derivative of Euler eq. error wrt consumption ------------------ 
            
            ! consumption with a little change for numerical derivative
            c_t0d = c_t0+1.0e-8
            
            ! budget constraint
            b_t1 = r_t0*(b_t0/(y_t0*p_t0*t_t0) + g_t0 - c_t0d)
            
            ! interpolation for b_t+1 and r_t
            jb(1) = min0(max0(1,int(1 + (b_t1-gridb(1))/((gridb(ngrib)-gridb(1))/(ngrib-1)))),ngrib-1); jb(2) = 1 + jb(1)
            wb(1) = (gridb(jb(2)) - b_t1)/(gridb(jb(2))-gridb(jb(1))); wb(2) = 1.0 - wb(1)
            
            if ( ngric > 1 ) then
                jc(1) = min0(max0(1,int(1 + (c_t0d-gridc(1))/((gridc(ngric)-gridc(1))/(ngric-1)))),ngric-1); jc(2) = 1 + jc(1)
                wc(1) = (gridc(jc(2)) - c_t0d)/(gridc(jc(2))-gridc(jc(1))); wc(2) = 1.0 - wc(1)
            else
                wc = (/0.5, 0.5/)
                jc = (/1, 1/)
            endif
            
            ! next periods' consumption and values
            c_t1(:,:,db) = 0.0; v_t1(:,:,db) = 0.0
            do ddb = 1, 2; do ddr = 1, 2; do ddc = 1, 2
            
                c_t1(:,:,db) = c_t1(:,:,db) + wb(ddb)*wr(ddr)*wc(ddc)*c_tf(:,:,jb(ddb),jr(ddr),jc(ddc))
                v_t1(:,:,db) = v_t1(:,:,db) + wb(ddb)*wr(ddr)*wc(ddc)*v_tf(:,:,jb(ddb),jr(ddr),jc(ddc))
            
            enddo; enddo; enddo
            
            ! AA = E[v^(1-\alp)]^(1/(1-\alp))
            AA(db) = 0.0
            do ddt = 1, ngrit
            
                AA(db) = AA(db) + mct(dt,ddt) * sum( mcyp(dyp,:)*(gridyp(:,1)*v_t1(:,ddt,db))**(1-alp_p) )
            
            enddo
            
            AA(db) = AA(db)**(1.0/(1.0-alp_p))
            
            ! euler eq. error
            eul_d(db) = 0.0
            do ddt = 1, ngrit
            
                eul_d(db) = eul_d(db) + mct(dt,ddt) * sum( mcyp(dyp,:)*( &
                    r_t0/(gridyp(:,2)*gridt(ddt)) * beta_p * ( (c_t1(:,ddt,db) - hab_p*c_t0d/gridyp(:,1))/(c_t0d - hab_p*gridc(dc)/gridyp(dyp,1)) )**(-sig_p) &
                    *(v_t1(:,ddt,db)/AA(db))**(sig_p-alp_p) * gridyp(:,1)**(-alp_p) ) )
            
            enddo
            
            eul_d(db) = 1.0 + phib*(b_t1/r_t0 - bst/rstar) - eul_d(db)
            
            ! numerical derivative of Euler eq. error wrt consumption
            deriv = (eul_d(db) - eul_re(db))/1.0e-8
            
            ! revise
            c_t0 = c_t0 - eul_re(db)/deriv
                
        enddo
        
        if (j == 1000 ) then
            print*, 'NG!!!'
        endif
        
        ! borrowing constraint
        if ( deaton_f == 1 .and. b_t1 < gridb(1) ) b_t1 = gridb(1)
        
        ! optimal debt, consumption, and Lagrange multiplier
        b_t(dyp,dt,db,dr,dc) = b_t1
        c_t(dyp,dt,db,dr,dc) = b_t0/(y_t0*p_t0*t_t0) + g_t0 - b_t1/r_t0
       
    enddo
    
end subroutine