
! This is for the term premium model for GFSR April 2017
! By Mitsuru Katagiri, 7/13/2017

program term04

    use common_var
    
    implicit none
    
!--------------------------------------------------------------------------------
!   define functions
!--------------------------------------------------------------------------------
    
    integer :: vfi_loop, multigrid, init_var, peri, dur, acc_iter, resid_vl(5), resid_cl(5)
    real(8) :: y_t0, p_t0, c_t0, b_t0, r_t0, b_t1, lag_t0, resid_v, resid_c, resid_cv, lb_c, ub_c
    
!--------------------------------------------------------------------------------
!   # of grids
!--------------------------------------------------------------------------------

    ! deaton mode or fiscal risk model. 1: deaton's model. 0: fiscal risk model 
    deaton_f = 0

!-------------- cases are set here !! -----------------
 
do robus_f  = 14, 15 !1: baseline, 2: high crra, 3: low crra, 4: high phip, 5: low phip, 6: stationary, 7: with elb, 8: L4L, 9: with phiy, 10: habit, 11: w/o trend inf
do cnt_case = 2, 2 !1: US, 2: UK, 3: GE, 4:JP
    
    ! flag for initial values. 1: use ini-file. 0: do not use ini-file 
    init_var = 0
    
    ! The robustness check is done only for the baseline case
    if ( robus_f > 8 .and. cnt_case > 1 .and. robus_f < 11 ) cycle
    !if ( robus_f < 10 .or. robus_f == 11 ) init_var = 1
    
    ! parameter values for baseline
    phiy  = 0.00    ! mp rule: income gap
    phir  = 0.80    ! mp rule: smoothing
    elb   = 0.00    ! effective lower bound (ELB) of interest rate
    hab_p = 0.00    ! habit
    alp_p = 40.0    ! crra
   
    select case ( cnt_case ) 
        
        ! mp rule: inflation
        case (1); phip  = 2.00; if (robus_f > 3 ) then; alp_p = 40.0; endif    
        case (2); phip  = 1.00; if (robus_f > 3 ) then; alp_p = 20.0; endif   
        case (3); phip  = 2.00; if (robus_f > 3 ) then; alp_p = 150.0; endif     
        case (4); phip  = 2.00; if (robus_f > 3 ) then; alp_p = 150.0; endif     
            
    end select

    ! Robustness checks
    select case ( robus_f ) 
        
        case (1) ;                robus = '01base/'
        case (2) ; alp_p = 150.0; robus = '02hris/'
        case (3) ; alp_p = 20.0;  robus = '03lris/'
        case (4) ; phip  = 3.00;  robus = '04hphp/'
        case (5) ; phip  = 1.00;  if ( cnt_case == 2 ) phip = 2.00; robus = '05lphp/'
        case (6) ;                robus = '06stat/'
        case (7) ; elb   = 1.00;  robus = '07elb_/'
        case (8) ; elb   = 1.00;  robus = '08l4l_/'
        case (9) ; phiy  = 0.50;  robus = '09phy_/'
        case (10); hab_p = 0.25;  robus = '10hab_/'
        case (11);                robus = '11woti/'
        case (12); elb   = 1.00;  robus = '12l4lh/'
        case (13); elb   = 1.00;  robus = '13l4ll/'
        case (14);                robus = '14ut87/'
        case (15);                robus = '15af87/'
            
    end select
        
    ! Set # of grids
    ngriy  = 5                  ! income growth
    ngrip  = 5                  ! inflation
    ngrig  = 5                  ! income gap
    ngriyp = ngriy*ngrip*ngrig  ! (y,p,g)
    ngrib  = 21                 ! bond
    !if ( robus_f == 7 .or. robus_f == 8 .or. robus_f == 10 .or. robus_f == 11 ) then; ngrit = 1; else; ngrit = 5 ; endif  ! trend inflation "ngrit = 1" means a fixed trend inflation    
    if ( robus_f > 3 .and. robus_f /= 10 ) then; ngrit = 1; else; ngrit = 5 ; endif  ! trend inflation "ngrit = 1" means a fixed trend inflation    
    if ( phir  == 0.0 ) then; ngrir = 1; else; ngrir = 21; endif  ! interest rate     
    if ( hab_p == 0.0 ) then; ngric = 1; else; ngric = 21; endif  ! consumption habit   
        
!--------------------------------------------------------------------------------
!   allocation and load 
!--------------------------------------------------------------------------------

    allocate ( gridyp(ngriyp,3), mcyp(ngriyp,ngriyp), mct(ngrit,ngrit), gridy(ngriy), gridp(ngrip), gridg(ngrig), gridb(ngrib), &
               gridr(ngrir), gridc(ngric), gridt(ngrit), rmat(ngriyp,ngrit,ngrir), AA(ngrib), rr_t0(ngrib) ) 
    allocate ( v_t(ngriyp,ngrit,ngrib,ngrir,ngric), c_t(ngriyp,ngrit,ngrib,ngrir,ngric), b_t(ngriyp,ngrit,ngrib,ngrir,ngric), &
               lag_t(ngriyp,ngrit,ngrib,ngrir,ngric), v_tf(ngriyp,ngrit,ngrib,ngrir,ngric), c_tf(ngriyp,ngrit,ngrib,ngrir,ngric), &
               c_t1(ngriyp,ngrit,ngrib), v_t1(ngriyp,ngrit,ngrib)  )
    
    ! ---------------- load shock files and parameters -----------------
    call load_files
    ! ------------------------------------------------------------------
    
    ! grid for bonds
    call lgridgen(0.0*bst, 2.0*bst, gridb, ngrib, 1)
        
    ! grid for interest rate
    if ( ngrir == 1 ) then; gridr = 1.0
    else 
        if ( elb >  0 ) call lgridgen(elb       , rstar**0.2 *((gridyp(ngriyp,2)/pistar)**phip *gridyp(ngriyp,3)**phiy *rstar)**0.8, gridr, ngrir, 2)
        if ( elb == 0 ) call lgridgen(rstar**0.0, rstar**0.2 *((gridyp(ngriyp,2)/pistar)**phip *gridyp(ngriyp,3)**phiy *rstar)**0.8, gridr, ngrir, 2)
    endif
    
    ! grid for consumption
    if ( ngric == 1 ) then; gridc = 1.0
    else
        
        ub_c = 1.15; lb_c = 0.90
        call lgridgen(lb_c, ub_c, gridc, ngric, 1)
    
    endif
    
    ! matrix for interest rate by Taylor rule
    do dr  = 1, ngrir
    do dyp = 1, ngriyp
    do dt  = 1, ngrit
    
        rmat(dyp,dt,dr) = max( (gridr(dr)/gridt(dt))**phir *((gridyp(dyp,2)/pistar)**phip *gridyp(dyp,3)**phiy *rstar)**(1.0-phir) , elb )
        
    enddo
    enddo
    enddo
    
!--------------------------------------------------------------------------------
!   Policy function iteration by Coleman(1990)
!--------------------------------------------------------------------------------

    ! Set initial values
    v_t   = 1.0   ! value function for t
    c_t   = 1.0   ! consumption for t
    b_t   = 0.0   ! bond for t
    lag_t = 0.0   ! Lagrange multiplier for t
    
    c_tf  = 1.0   ! consumption for t+1 in the previous iteration
    v_tf  = 6.0   ! value function for t+1 in the previous iteration
    
    if ( init_var == 1 ) then
    
        !open (90, file=cnt//robus//'c_ini.txt')     ! READ shock grids
        open (90, file=cnt//'01base\c_ini.txt')     ! READ shock grids
            read (90, *), c_tf
        close (90)

        !open (90, file=cnt//robus//'v_ini.txt')     ! READ shock grids
        open (90, file=cnt//'01base\v_ini.txt')     ! READ shock grids
            read (90, *), v_tf
        close (90)
    
    endif
    
    do vfi_loop = 1, 5000  ! time iteration
 !    do vfi_loop = 1, 1  ! time iteration
    
        do dt  = 1, ngrit
        do dr  = 1, ngrir
        do dc  = 1, ngric
        do dyp = 1, ngriyp
            
            ! compute optimal consumption
            call opt_con
            
            !do db = 1, ngrib
            !    
            !    call euler_eq(c_t(dyp,dt,db,dr,dc),lag_t0)
            !    lag_t(dyp,dt,db,dr,dc) = lag_t0
            !
            !enddo
            
        enddo
        enddo
        enddo
        enddo
        
        ! compute v_t based on c_t and b_t
        call accel
        
        !-------- revision ---------
        
        ! residual for the policy functions
        resid_vl = maxloc(abs(v_t/v_tf-1.0))
        resid_cl = maxloc(abs(c_t/c_tf-1.0))
        resid_v  = maxval(abs(v_t/v_tf-1.0))
        resid_c  = maxval(abs(c_t/c_tf-1.0))
        resid_cv = max(resid_c, resid_v)
        
        ! converge?
        if ( mod(vfi_loop,10) == 0 .or. vfi_loop == 1 ) then
        
            print*, vfi_loop/10
            print*, resid_v, resid_c
            print*, v_t(resid_vl(1),resid_vl(2),resid_vl(3),resid_vl(4),resid_vl(5)), v_tf(resid_vl(1),resid_vl(2),resid_vl(3),resid_vl(4),resid_vl(5))
            print*, c_t(resid_cl(1),resid_cl(2),resid_cl(3),resid_cl(4),resid_cl(5)), c_tf(resid_cl(1),resid_cl(2),resid_cl(3),resid_cl(4),resid_cl(5))
        
        endif

        if ( resid_cv < 1.0e-4 .or. ( vfi_loop > 100 .and. resid_c < 1.0e-4 ) ) then
        
            print*, 'PFI Done!!'
            exit
        
        endif
        
        !if ( resid_v > 1.0e-2 .and. vfi_loop > 10 ) goto 222
        ! in this case, the policy fucntion is diverging...
        
        v_tf = adjspeed*v_t + (1-adjspeed)*v_tf
        c_tf = adjspeed*c_t + (1-adjspeed)*c_tf
        
        if ( resid_v > 1.0e-4 ) then
        
            !do acc_iter = 1, 1
            do acc_iter = 1, 30
             
                call accel
                v_tf = adjspeed*v_t + (1-adjspeed)*v_tf
                
            enddo
        
        endif
        
    enddo !vfi_loop
    
    
!------------------ compute term structure ---------------------
    
    ! weight for interpolation wrt optimal bond holdings
    allocate ( jbs(2,ngriyp,ngrit,ngrib,ngrir,ngric), wbs(2,ngriyp,ngrit,ngrib,ngrir,ngric), jcs(2,ngriyp,ngrit,ngrib,ngrir,ngric), & 
               wcs(2,ngriyp,ngrit,ngrib,ngrir,ngric), jrs(2,ngriyp,ngrit,ngrir), wrs(2,ngriyp,ngrit,ngrir) )   

    do dr  = 1, ngrir
    do dyp = 1, ngriyp
    do dt  = 1, ngrit
        
        call inter_weight(log(rmat(dyp,dt,dr)), log(gridr), wr, jr, ngrir) 
        
        jrs(:,dyp,dt,dr) = jr
        wrs(:,dyp,dt,dr) = wr
        
        do db  = 1, ngrib
        do dc  = 1, ngric
            
            call inter_weight(b_t(dyp,dt,db,dr,dc), gridb, wb, jb, ngrib)
            
            jbs(:,dyp,dt,db,dr,dc) = jb
            wbs(:,dyp,dt,db,dr,dc) = wb
            
            call inter_weight(c_t(dyp,dt,db,dr,dc), gridc, wc, jc, ngric)
            
            jcs(:,dyp,dt,db,dr,dc) = jc
            wcs(:,dyp,dt,db,dr,dc) = wc

        enddo
        enddo
        
    enddo    
    enddo
    enddo

    allocate (   ap_t(ngriyp,ngrit,ngrib,ngrir,ngric),   ap_tn(ngriyp,ngrit,ngrib,ngrir,ngric), &
                rap_t(ngriyp,ngrit,ngrib,ngrir,ngric),  rap_tn(ngriyp,ngrit,ngrib,ngrir,ngric), &
                ap_tf(ngriyp,ngrit,ngrib,ngrir,ngric),  ap_tnf(ngriyp,ngrit,ngrib,ngrir,ngric), &
               rap_tf(ngriyp,ngrit,ngrib,ngrir,ngric), rap_tnf(ngriyp,ngrit,ngrib,ngrir,ngric), &
               ap_t1(ngriyp,ngrit,ngrib), ap_tn1(ngriyp,ngrit,ngrib), rap_t1(ngriyp,ngrit,ngrib), rap_tn1(ngriyp,ngrit,ngrib), &
               sdf_t1(ngriyp,ngrit,ngrib), rsdf_t1(ngriyp,ngrit,ngrib) )
    
    ap_tf = 1.0; ap_tnf = 1.0; rap_tf = 1.0; rap_tnf = 1.0
                 
    ! nominal and real yield curve by tenor(1,20,40) and types(risk-averse, risk-neutral)
    allocate (  yie0_n(ngriyp,ngrit,ngrib,ngrir,ngric,2),  yie0_r(ngriyp,ngrit,ngrib,ngrir,ngric,2), &
                yie1_n(ngriyp,ngrit,ngrib,ngrir,ngric,2),  yie2_n(ngriyp,ngrit,ngrib,ngrir,ngric,2), &
                yie3_n(ngriyp,ngrit,ngrib,ngrir,ngric,2),  yie4_n(ngriyp,ngrit,ngrib,ngrir,ngric,2), &
                yie5_n(ngriyp,ngrit,ngrib,ngrir,ngric,2),  yie5_r(ngriyp,ngrit,ngrib,ngrir,ngric,2), &
               yie10_n(ngriyp,ngrit,ngrib,ngrir,ngric,2), yie10_r(ngriyp,ngrit,ngrib,ngrir,ngric,2)  )
                 
    ! their intial values are one because all asset prices with maturity=0 is equal to one.
    do dur = 1, 40
    !do dur = 1, 1
        
        ap_t = 0.0; ap_tn = 0.0; rap_t = 0.0; rap_tn = 0.0
        
        do dyp = 1, ngriyp
        do dr  = 1, ngrir
        do dt  = 1, ngrit
        do dc  = 1, ngric
        
            c_t1 = 0.0; v_t1 = 0.0; ap_t1 = 0.0; ap_tn1 = 0.0; rap_t1 = 0.0; rap_tn1 = 0.0
            
!$omp parallel do private(jb,wb,jc,wc,jr,wr,ddb,ddc,ddr,ddt)
        do db  = 1, ngrib
            
            ! interpolation for bonds
            jb = jbs(:,dyp,dt,db,dr,dc)
            wb = wbs(:,dyp,dt,db,dr,dc)
            
            ! interpolation for consumption
            jc = jcs(:,dyp,dt,db,dr,dc)
            wc = wcs(:,dyp,dt,db,dr,dc)
            
            ! interpolation for interest rate
            jr = jrs(:,dyp,dt,dr)
            wr = wrs(:,dyp,dt,dr)
            
            do ddb = 1,2; do ddc = 1,2; do ddr = 1,2
                
                c_t1(:,:,db) = c_t1(:,:,db) + wb(ddb)*wr(ddr)*wc(ddc)*c_tf(:,:,jb(ddb),jr(ddr),jc(ddc)) 
                v_t1(:,:,db) = v_t1(:,:,db) + wb(ddb)*wr(ddr)*wc(ddc)*v_tf(:,:,jb(ddb),jr(ddr),jc(ddc)) 
                
                ! asset prices for the next period
                ap_t1(:,:,db)   = ap_t1(:,:,db)   + wb(ddb)*wr(ddr)*wc(ddc)*   ap_tf(:,:,jb(ddb),jr(ddr),jc(ddc)) 
                ap_tn1(:,:,db)  = ap_tn1(:,:,db)  + wb(ddb)*wr(ddr)*wc(ddc)*  ap_tnf(:,:,jb(ddb),jr(ddr),jc(ddc)) 
                rap_t1(:,:,db)  = rap_t1(:,:,db)  + wb(ddb)*wr(ddr)*wc(ddc)*  rap_tf(:,:,jb(ddb),jr(ddr),jc(ddc)) 
                rap_tn1(:,:,db) = rap_tn1(:,:,db) + wb(ddb)*wr(ddr)*wc(ddc)* rap_tnf(:,:,jb(ddb),jr(ddr),jc(ddc)) 
                      
            enddo; enddo; enddo

            ! AA = E[v^(1-\alp)]^(1/(1-\alp))
            AA(db) = 0.0
            do ddt = 1, ngrit
            
                AA(db) = AA(db) + mct(dt,ddt) * sum( mcyp(dyp,:)*(gridyp(:,1)*v_t1(:,ddt,db))**(1-alp_p) )
            
            enddo
            AA(db) = AA(db)**(1.0/(1.0-alp_p))
            
            do ddt = 1, ngrit
                
                ! nominal STD
                sdf_t1(:,ddt,db)  = 1/gridyp(:,2)*beta_p &
                    *((c_t1(:,ddt,db)-hab_p*c_t(dyp,dt,db,dr,dc)/gridyp(:,1))/(c_t(dyp,dt,db,dr,dc)-hab_p*gridc(dc)/gridyp(dyp,1)))**(-sig_p) &
                    *(v_t1(:,ddt,db)/AA(db))**(sig_p-alp_p) * gridyp(:,1)**(-alp_p)
           
                ! real STD
                rsdf_t1(:,ddt,db) = beta_p &
                    *((c_t1(:,ddt,db)-hab_p*c_t(dyp,dt,db,dr,dc)/gridyp(:,1))/(c_t(dyp,dt,db,dr,dc)-hab_p*gridc(dc)/gridyp(dyp,1)))**(-sig_p) &
                    *(v_t1(:,ddt,db)/AA(db))**(sig_p-alp_p) * gridyp(:,1)**(-alp_p)
                
            enddo
            
            ! real interest rate 
            rr_t0(db) = 0.0
            do ddt = 1, ngrit
                
                rr_t0(db) = rr_t0(db) + mct(dt,ddt)* sum( mcyp(dyp,:)*rsdf_t1(:,ddt,db) )
                
            enddo
            rr_t0(db) = 1.0/rr_t0(db)
            
            do ddt = 1, ngrit

                ! nominal bond price
                ap_t(dyp,dt,db,dr,dc)   =  ap_t(dyp,dt,db,dr,dc) + mct(dt,ddt)*sum( mcyp(dyp,:)*(sdf_t1(:,ddt,db)*ap_t1(:,ddt,db)/gridt(ddt)**dur) ) &
                    !/(1-lag_t(dyp,dt,db,dr,dc)+phib*(b_t(dyp,dt,db,dr,dc)/rmat(dyp,dt,dr)-bst/rstar)) 
                    /(1+phib*(b_t(dyp,dt,db,dr,dc)/rmat(dyp,dt,dr)-bst/rstar)) 
            
                ! nominal bond price for risk-neutral agents
                ap_tn(dyp,dt,db,dr,dc)  =  ap_tn(dyp,dt,db,dr,dc) + mct(dt,ddt)* 1.0/rmat(dyp,dt,dr)* sum( mcyp(dyp,:)*ap_tn1(:,ddt,db)/gridt(ddt)**(dur-1.0) ) 
                    !/(1-lag_t(dyp,dt,db,dr,dc)) 
            
                ! real bond price
                rap_t(dyp,dt,db,dr,dc)  =  rap_t(dyp,dt,db,dr,dc) + mct(dt,ddt)* sum( mcyp(dyp,:)*(rsdf_t1(:,ddt,db)*rap_t1(:,ddt,db)) ) &
                    !/(1-lag_t(dyp,dt,db,dr,dc)+phib*(b_t(dyp,dt,db,dr,dc)/rmat(dyp,dt,dr)-bst/rstar)) 
                    /(1+phib*(b_t(dyp,dt,db,dr,dc)/rmat(dyp,dt,dr)-bst/rstar)) 
            
                ! real bond price for risk-neutral agents
                rap_tn(dyp,dt,db,dr,dc) = rap_tn(dyp,dt,db,dr,dc) + mct(dt,ddt)* 1.0/rr_t0(db) * sum( mcyp(dyp,:)*rap_tn1(:,ddt,db) ) &
                    !/(1-lag_t(dyp,dt,db,dr,dc)) 
                    /(1+phib*(b_t(dyp,dt,db,dr,dc)/rmat(dyp,dt,dr)-bst/rstar)) 
            
            enddo
            
        enddo
        
        enddo
        enddo
        enddo
        enddo
        
        ! the price of n-year bond will be used for the price of n-1 year bond in the next iteration
        ap_tf = ap_t; ap_tnf = ap_tn; rap_tf = rap_t; rap_tnf = rap_tn 
            
        if ( dur == 1 ) then

            yie0_n(:,:,:,:,:,1) = (1/ap_t)**(1.0/dur);  yie0_n(:,:,:,:,:,2) = (1/ap_tn)**(1.0/dur)
            yie0_r(:,:,:,:,:,1) = (1/rap_t)**(1.0/dur); yie0_r(:,:,:,:,:,2) = (1/rap_tn)**(1.0/dur)
            print*, 'policy rate Done!!'
            
        elseif ( dur == 4 ) then

            yie1_n(:,:,:,:,:,1) = (1/ap_t)**(1.0/dur);  yie1_n(:,:,:,:,:,2) = (1/ap_tn)**(1.0/dur)
            
        elseif ( dur == 8 ) then

            yie2_n(:,:,:,:,:,1) = (1/ap_t)**(1.0/dur);  yie2_n(:,:,:,:,:,2) = (1/ap_tn)**(1.0/dur)
            
        elseif ( dur == 12 ) then

            yie3_n(:,:,:,:,:,1) = (1/ap_t)**(1.0/dur);  yie3_n(:,:,:,:,:,2) = (1/ap_tn)**(1.0/dur)
            
        elseif ( dur == 16 ) then

            yie4_n(:,:,:,:,:,1) = (1/ap_t)**(1.0/dur);  yie4_n(:,:,:,:,:,2) = (1/ap_tn)**(1.0/dur)
            
        elseif ( dur == 20 ) then

            yie5_n(:,:,:,:,:,1) = (1/ap_t)**(1.0/dur);  yie5_n(:,:,:,:,:,2) = (1/ap_tn)**(1.0/dur)
            yie5_r(:,:,:,:,:,1) = (1/rap_t)**(1.0/dur); yie5_r(:,:,:,:,:,2) = (1/rap_tn)**(1.0/dur)
            print*, '5y Done!!'
            
        elseif ( dur == 40 ) then

            yie10_n(:,:,:,:,:,1) = (1/ap_t)**(1.0/dur);  yie10_n(:,:,:,:,:,2) = (1/ap_tn)**(1.0/dur)
            yie10_r(:,:,:,:,:,1) = (1/rap_t)**(1.0/dur); yie10_r(:,:,:,:,:,2) = (1/rap_tn)**(1.0/dur)
            print*, '10y Done!!'
            
        endif
         
    enddo


!--------------------------------------------------------------------------------
!   Save the results
!--------------------------------------------------------------------------------

    open (90, file=cnt//robus//'c_ini.txt')     ! Output the result to .TXT file
            write (90, *), c_t
    close (90)

    open (90, file=cnt//robus//'v_ini.txt')     ! Output the result to .TXT file
            write (90, *), v_t
    close (90)

    open (90, file=cnt//robus//'b_t.txt')     ! Output the result to .TXT file
        do dc  = 1, ngric; do dr  = 1, ngrir; do db  = 1, ngrib; do dyp = 1, ngriyp
            
            write (90, '(1000F30.8)'), b_t(dyp,:,db,dr,dc)
            
        enddo; enddo; enddo; enddo
    close (90)
    
    open (90, file=cnt//robus//'c_t.txt')     ! Output the result to .TXT file
        do dc  = 1, ngric; do dr  = 1, ngrir; do db  = 1, ngrib; do dyp = 1, ngriyp
            
            write (90, '(1000F30.8)'), c_t(dyp,:,db,dr,dc)
            
        enddo; enddo; enddo; enddo
    close (90)
    
    open (90, file=cnt//robus//'yie0_n.txt')     ! Output the result to .TXT file
        do dt  = 1, 2; do dc  = 1, ngric; do dr  = 1, ngrir; do db  = 1, ngrib; do dyp = 1, ngriyp
            
            write (90, '(1000F30.8)'), yie0_n(dyp,:,db,dr,dc,dt)
            
        enddo; enddo; enddo; enddo; enddo
    close (90)
    
    open (90, file=cnt//robus//'yie0_r.txt')     ! Output the result to .TXT file
        do dt  = 1, 2; do dc  = 1, ngric; do dr  = 1, ngrir; do db  = 1, ngrib; do dyp = 1, ngriyp
            
            write (90, '(1000F30.8)'), yie0_r(dyp,:,db,dr,dc,dt)
            
        enddo; enddo; enddo; enddo; enddo
    close (90)
    
    open (90, file=cnt//robus//'yie1_n.txt')     ! Output the result to .TXT file
        do dt  = 1, 2; do dc  = 1, ngric; do dr  = 1, ngrir; do db  = 1, ngrib; do dyp = 1, ngriyp
            
            write (90, '(1000F30.8)'), yie1_n(dyp,:,db,dr,dc,dt)
            
        enddo; enddo; enddo; enddo; enddo
    close (90)
    
    open (90, file=cnt//robus//'yie2_n.txt')     ! Output the result to .TXT file
        do dt  = 1, 2; do dc  = 1, ngric; do dr  = 1, ngrir; do db  = 1, ngrib; do dyp = 1, ngriyp
            
            write (90, '(1000F30.8)'), yie2_n(dyp,:,db,dr,dc,dt)
            
        enddo; enddo; enddo; enddo; enddo
    close (90)
    
    open (90, file=cnt//robus//'yie3_n.txt')     ! Output the result to .TXT file
        do dt  = 1, 2; do dc  = 1, ngric; do dr  = 1, ngrir; do db  = 1, ngrib; do dyp = 1, ngriyp
            
            write (90, '(1000F30.8)'), yie3_n(dyp,:,db,dr,dc,dt)
            
        enddo; enddo; enddo; enddo; enddo
    close (90)
    
    open (90, file=cnt//robus//'yie4_n.txt')     ! Output the result to .TXT file
        do dt  = 1, 2; do dc  = 1, ngric; do dr  = 1, ngrir; do db  = 1, ngrib; do dyp = 1, ngriyp
            
            write (90, '(1000F30.8)'), yie4_n(dyp,:,db,dr,dc,dt)
            
        enddo; enddo; enddo; enddo; enddo
    close (90)
    
    open (90, file=cnt//robus//'yie5_n.txt')     ! Output the result to .TXT file
        do dt  = 1, 2; do dc  = 1, ngric; do dr  = 1, ngrir; do db  = 1, ngrib; do dyp = 1, ngriyp
            
            write (90, '(1000F30.8)'), yie5_n(dyp,:,db,dr,dc,dt)
            
        enddo; enddo; enddo; enddo; enddo
    close (90)
    
    open (90, file=cnt//robus//'yie5_r.txt')     ! Output the result to .TXT file
        do dt  = 1, 2; do dc  = 1, ngric; do dr  = 1, ngrir; do db  = 1, ngrib; do dyp = 1, ngriyp
            
            write (90, '(1000F30.8)'), yie5_r(dyp,:,db,dr,dc,dt)
            
        enddo; enddo; enddo; enddo; enddo
    close (90)
    
    open (90, file=cnt//robus//'yie10_n.txt')     ! Output the result to .TXT file
        do dt  = 1, 2; do dc  = 1, ngric; do dr  = 1, ngrir; do db  = 1, ngrib; do dyp = 1, ngriyp
            
            write (90, '(1000F30.8)'), yie10_n(dyp,:,db,dr,dc,dt)
            
        enddo; enddo; enddo; enddo; enddo
    close (90)
    
    open (90, file=cnt//robus//'yie10_r.txt')     ! Output the result to .TXT file
        do dt  = 1, 2; do dc  = 1, ngric; do dr  = 1, ngrir; do db  = 1, ngrib; do dyp = 1, ngriyp
            
            write (90, '(1000F30.8)'), yie10_r(dyp,:,db,dr,dc,dt)
            
        enddo; enddo; enddo; enddo; enddo
    close (90)
    
    open (90, file=cnt//robus//'gridc.txt')     ! Output the result to .TXT file
            write (90, '(1000F30.8)'), gridc
    close (90)
    
    open (90, file=cnt//robus//'gridb.txt')     ! Output the result to .TXT file
            write (90, '(1000F30.8)'), gridb
    close (90)
   
    open (90, file=cnt//robus//'gridr.txt')     ! Output the result to .TXT file
            write (90, '(1000F30.8)'), gridr
    close (90)
    
    deallocate ( yie0_n, yie0_r, yie1_n, yie2_n, yie3_n, yie4_n, yie5_n, yie5_r, yie10_n, yie10_r )
    deallocate ( ap_t, ap_tn, rap_t, rap_tn, ap_tf, ap_tnf, rap_tf, rap_tnf, ap_t1, ap_tn1, rap_t1, rap_tn1 )
    deallocate ( jbs, wbs, jcs, wcs, jrs, wrs )  
    
    deallocate ( gridyp, mcyp, mct, gridy, gridp, gridg, gridt, gridb, gridr, gridc, rmat, AA, rr_t0 ) 
    deallocate ( v_t, c_t, b_t, lag_t, v_tf, c_tf, c_t1, v_t1, sdf_t1, rsdf_t1 )    
                
enddo

    print*, cnt_case
    
enddo

222 print*, 'end...'
   
endprogram
    
