// This is estimation income process
// Written by Mitsuru Katagiri 2017/6/21

% NOTE: if you want to estimate with trend inflation, please:
% 1. change the observation variable from "pi_obs" to "dpi_obs"
% 2. add "ar_ti" as an estimated param

%-----------------------------------------------------------------------------------------------
% 0. Housekeeping 
%-----------------------------------------------------------------------------------------------

close all;

%-----------------------------------------------------------------------------------------------
% 1. Defining variables
%-----------------------------------------------------------------------------------------------

var gU piU yU rU tiU;
var g_obs pi_obs dpi_obs r_obs; 
varexo e_pi e_g e_y e_ti e_r ; %e_yi;   

parameters ar_pp ar_pg ar_py ar_gg ar_gp ar_gy ar_yg ar_yp ar_yy ar_ti;
parameters phi_r phi_p phi_y;
parameters var_g var_pi var_y var_r varratio_p ;% var_yi ; 

%-----------------------------------------------------------------------------------------------
% 2. Set parameter values
%-----------------------------------------------------------------------------------------------

  ar_gg = 0.00;
  ar_gp = 0.00;
  ar_gy = 0.00;
  ar_pg = 0.00;
  ar_pp = 0.50; 
  ar_py = 0.00; 
  ar_yg = 0.00;
  ar_yp = 0.00; 
  ar_yy = 0.60; 
  ar_ti = 0.98;
  
  phi_r = 0.8;
  phi_p = 2.0;
  phi_y = 0.5;

  var_g   =  0.60;
  var_y   =  0.60;
  var_r   =  0.60;
  var_pi  =  0.10;
  varratio_p = 0.01;
  var_yi  =  0.10;
 
%------------------------------------------------------------------------------------------------
% 3. Model
%------------------------------------------------------------------------------------------------

model;
    
  gU  = ar_gg *gU(-1) + ar_gp *piU(-1) + ar_gy *yU(-1) + var_g ^(1/2)*e_g ;  
  piU = ar_pg *gU(-1) + ar_pp *piU(-1) + ar_py *yU(-1) + var_pi^(1/2)*e_pi; 
  yU  = ar_yg *gU(-1) + ar_yp *piU(-1) + ar_yy *yU(-1) + var_y ^(1/2)*e_y ;
  tiU = ar_ti *tiU(-1)+ varratio_p*var_pi^(1/2)*e_ti;
  rU  = phi_r*rU(-1) + (1-phi_r)*((1+phi_p)*piU + phi_y*yU) ; 
%   gU  = ar_gg *gU(-1)  + var_g^(1/2) *e_g ;  
%   piU = ar_pp *piU(-1) + var_pi^(1/2)*e_pi; 
  
  % Observation equations
  %-------------------------------------------
  
  g_obs  = gU + yU - yU(-1) ;% + e_yi - e_yi(-1) ; 
  dpi_obs = piU - piU(-1) + tiU; 
  pi_obs = piU ;
  r_obs  = rU + var_r^(1/2)*e_r ;
  
end;

%-------------------------------------------------------------------------
% 4. Priors
%-------------------------------------------------------------------------

varobs dpi_obs g_obs  ;%r_obs    

initval;

  piU = 0.0;
  gU  = 0.0;
  yU  = 0.0;
  rU  = 0.0;
  tiU = 0.0;
  
  g_obs   = 0.0;
  dpi_obs = 0.0;
  pi_obs  = 0.0;
  r_obs   = 0.0;
  
end;

steady;

shocks; 

  var e_g  = 1.0;
  var e_pi = 1.0;
  var e_y  = 1.0;
  var e_r  = 1.0;
  var e_ti = 1.0;
%   var e_yi = 1.0;

end;
  
% estimated_params;
%   
%   ar_gg , 0.30;
%   ar_pp , 0.50;  
%   ar_yy , 0.00;  
%   
%   phi_r , 0.8;
%   phi_p , 2.0;
%   phi_y , 0.5;
% 
%   var_y , 0.50;
%   var_g , 0.20; 
%   var_pi, 0.20; 
%   
%   corr e_g, e_pi, -0.50;
%   corr e_y, e_pi, 0.30;
% 
% end;

estimated_params;
  
  ar_gg , beta_pdf,   0.50, 0.25;
  ar_pp , beta_pdf,   0.50, 0.25;
  ar_py , uniform_pdf, 0.0, (1/3)^0.5;
  ar_yp , uniform_pdf, 0.0, (1/3)^0.5; 
  ar_yy , beta_pdf,   0.80, 0.05;  
  ar_ti , beta_pdf,   0.95, 0.03;
  
  var_y , inv_gamma_pdf, 0.4, inf;
  var_g , inv_gamma_pdf, 0.3, inf; 
  var_pi, inv_gamma_pdf, 0.3, inf; 
  
  corr e_g, e_pi, normal_pdf, -0.51, 1.0;
  corr e_y, e_pi, normal_pdf,  0.31, 1.0;

end;

%-------------------------------------------------------------------------
% 5. Estimation
%-------------------------------------------------------------------------

estimation(
datafile = inf_inc_us
%,first_obs=73
%,nobs = 60
,order =1
,mode_check
,mh_replic = 200000
% ,mh_jscale = 0.76 ); % without ar_yp
,mh_jscale = 0.70 ); 
% stoch_simul;



