// This is estimation income process
// Written by Mitsuru Katagiri 2017/6/21

% NOTE: if you want to estimate with trend inflation, please:
% 1. change the observation variable from "pi_obs" to "dpi_obs"
% 2. add "ar_ti" as an estimated param

%-----------------------------------------------------------------------------------------------
% 0. Housekeeping 
%-----------------------------------------------------------------------------------------------

close all;

%-----------------------------------------------------------------------------------------------
% 1. Defining variables
%-----------------------------------------------------------------------------------------------

var piU yU rU tiU;
var y_obs pi_obs dpi_obs r_obs; 
varexo e_pi e_y e_ti e_r;   

parameters ar_pp ar_py ar_yp ar_yy ar_ti;
parameters phi_r phi_p phi_y;
parameters var_pi var_y var_r varratio_p; 

%-----------------------------------------------------------------------------------------------
% 2. Set parameter values
%-----------------------------------------------------------------------------------------------

  ar_pp = 0.50; 
  ar_py = 0.00; 
  ar_yp = 0.00; 
  ar_yy = 0.60; 
  ar_ti = 0.98;
  
  phi_r = 0.8;
  phi_p = 2.0;
  phi_y = 0.5;

  var_y   =  0.60;
  var_r   =  0.60;
  var_pi  =  0.10;
  varratio_p = 0.01;
 
%------------------------------------------------------------------------------------------------
% 3. Model
%------------------------------------------------------------------------------------------------

model;
    
  piU = ar_pp *piU(-1) + ar_py *yU(-1) + var_pi^(1/2)*e_pi; 
  yU  = ar_yp *piU(-1) + ar_yy *yU(-1) + var_y ^(1/2)*e_y ;
  tiU = ar_ti *tiU(-1)+ varratio_p*var_pi^(1/2)*e_ti;
  rU  = phi_r*rU(-1) + (1-phi_r)*((1+phi_p)*piU + phi_y*yU) ; 
  
  % Observation equations
  %-------------------------------------------
  
  y_obs  = yU ; 
  dpi_obs = piU - piU(-1) + tiU; 
  pi_obs = piU ;
  r_obs  = rU + var_r^(1/2)*e_r ;
  
end;

%-------------------------------------------------------------------------
% 4. Priors
%-------------------------------------------------------------------------

varobs dpi_obs y_obs  ;%r_obs    

initval;

  piU = 0.0;
  yU  = 0.0;
  rU  = 0.0;
  tiU = 0.0;
  
  y_obs   = 0.0;
  dpi_obs = 0.0;
  pi_obs  = 0.0;
  r_obs   = 0.0;
  
end;

steady;

shocks; 

  var e_pi = 1.0;
  var e_y  = 1.0;
  var e_r  = 1.0;
  var e_ti = 1.0;

end;
  
% estimated_params;
%   
%   ar_gg , 0.30;
%   ar_pp , 0.50;  
%   ar_yy , 0.00;  
%   
%   phi_r , 0.8;
%   phi_p , 2.0;
%   phi_y , 0.5;
% 
%   var_y , 0.50;
%   var_g , 0.20; 
%   var_pi, 0.20; 
%   
%   corr e_g, e_pi, -0.50;
%   corr e_y, e_pi, 0.30;
% 
% end;

estimated_params;
  
  ar_pp , beta_pdf,   0.5, 0.25;
  ar_py , uniform_pdf, 0.0, (1/3)^0.5;
  ar_yp , uniform_pdf, 0.0, (1/3)^0.5; 
  ar_yy , beta_pdf,   0.50, 0.25;  
  ar_ti , beta_pdf,   0.95, 0.03;
  
  var_y , inv_gamma_pdf, 0.4, inf;
  var_pi, inv_gamma_pdf, 0.3, inf; 
  
  corr e_y, e_pi, uniform_pdf, 0.0, (1/3)^0.5;

end;

%-------------------------------------------------------------------------
% 5. Estimation
%-------------------------------------------------------------------------

estimation(
datafile = inf_inc_uk
%,first_obs=73
%,nobs = 60
,order =1
,mode_check
,mh_replic = 200000
,mh_jscale = 0.90 ); 

% stoch_simul;



